"""Check the latest version at https://pypi.org/project/baato/"""
__version__ = "1.0.1"
__pypi_username__ = "baato"
__pypi_packagename__ = "baato"
__github_username__ = "baato"
__github_reponame__ = "python-client"
__baato_endpoint__ = "https://api.baato.io/api"
__baato_version__ = "v1"
